import threading
from prompt_toolkit import prompt

def run_prompt_in_thread(data):
    result = None

    def target():
        nonlocal result
        result = prompt(f"Are you sure you want to continue with {data}? [y/n]: ")

    thread = threading.Thread(target=target)
    thread.start()
    thread.join()

    return result.lower() in ['y', 'yes']





from typing import List, Tuple, Optional
import click, threading
from prompt_toolkit import prompt
from prompt_toolkit.shortcuts import radiolist_dialog
from configstore.configstore import Config
from prompt_toolkit.styles import Style

def get_profiles_for_store(config_store):
    CONFIG = Config(config_store)
    return [PROFILE for PROFILE in CONFIG.PROFILES if 'latest' not in PROFILE]

def display_profile_menu_thread(profiles):
    def run_dialog():
        nonlocal selected_profile_id
        style = Style.from_dict({
            'dialog': 'bg:#4B4B4B',
            'dialog.body': 'bg:#242424 fg:#FFFFFF',
            'dialog.title': 'bg:#00aa00',
            'radiolist': 'bg:#1C1C1C fg:#FFFFFF',
            'button': 'bg:#528B8B',
            'button.focused': 'bg:#00aa00',
        })

        # Assuming profiles is a list of dicts, each with 'id' and 'name' keys
        values = [(profile, profile) for profile in profiles]


        selected_profile_id = radiolist_dialog(
            title="Select Profile",
            text="Choose a profile:",
            values=values,
            style=style
        ).run()

    selected_profile_id = None
    dialog_thread = threading.Thread(target=run_dialog)
    dialog_thread.start()
    dialog_thread.join()
    return selected_profile_id

def switch_profile(config_store, verbose: bool = False) -> Tuple[str, Optional[str]]:
    """
    Fetch a list of profiles, present them to the user, and switch to the selected profile.

    Args:
        config_store (str): The config store name to fetch profiles from.
        verbose (bool): If True, print additional info.

    Returns:
        Tuple[str, Optional[str]]: A tuple containing the ID of the active profile, and an error message if any.
    """

    profiles = get_profiles_for_store(config_store)  # Fetch profiles for the given config store

    # Check if the profiles list is empty
    if not profiles:
        return "", "No profiles found for the given config store!"

    selected_profile_id = display_profile_menu_thread(profiles)

    # Check if a profile was selected
    if selected_profile_id:
        # Logic to switch to the selected profile
        # This is where you would put the logic to actually switch profiles based on your application's needs
        return f"{selected_profile_id}", None
    else:
        return "", "Profile selection failed"

def select_profile(config_store):
    config_store = ''.join(config_store)
    result, error = switch_profile(config_store, verbose=True)
    if error:
        return 'DEFAULT'
    else:
        return result




from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.keys import Keys
from goatshell.ui import getLayout
from goatshell.confirm import run_prompt_in_thread
from goatshell.switch_profile import select_profile as select_profile
import os
from configstore.configstore import Config

class CustomKeyBindings(KeyBindings):
    def __init__(self, app, goatshell_instance):
        super().__init__()
        self.app = app  # Store the Application reference
        self.goatshell_instance = goatshell_instance

        @self.add(Keys.F6)
        def _(event):
            if self.goatshell_instance.prefix == 'goat':
                # No need to assign here if toggle_config_store_profile updates the Goatshell instance directly
                self.profile = self.goatshell_instance.toggle_config_store_profile()
                self.goatshell_instance.profile = self.profile
                update_latest_profile(self.goatshell_instance.current_config_store, self.goatshell_instance.profile)
            # Ensure getLayout() and prompt regeneration use the updated Goatshell state
            getLayout()
            self.app.invalidate()
            event.app.exit(result='re-prompt')

        @self.add(Keys.F7)
        def handle_f7(event):
            print()
            cloud_provider = self.goatshell_instance.prefix
            if cloud_provider != 'goat':
                os_command = f"goat {cloud_provider} extract commands"
                try:
                    result = self.goatshell_instance.execute_os_command(os_command)
                    if result == "failure":
                        print("INFO: failed to execute the command.")
                    else:
                        print(f"INFO: executed the command {os_command}")
                except:
                    pass
            else:
                data = f'extracting command tree for {cloud_provider}'
                confirmation = run_prompt_in_thread(data)
                if confirmation is True:
                    print("User confirmed:", confirmation)
                    directory = os.path.dirname(os.path.abspath(__file__))
                    os_command = f"python3 {directory}/fetch_goat.py"
                    try:
                        os.system(os_command)
                    except Exception as e:
                       print(f"Failed to execute command: {e}")
                else:
                    print("Action cancelled by user.")

            event.app.invalidate()
            event.app.exit(result='re-prompt')  # Signal to re-prompt.

        @self.add(Keys.F8)
        def handle_f8(event):
            goatshell_instance.switch_to_next_provider()
            if self.goatshell_instance.prefix == 'az':
                sub_id, sub_name = self.goatshell_instance.fetch_current_azure_subscription()
                if sub_id and sub_name:
                    self.goatshell_instance.profile = sub_name
                else:
                    self.profile = 'DEFAULT'
            else:
                self.profile = self.goatshell_instance.get_profile(self.goatshell_instance.prefix)
            getLayout()
            self.app.invalidate()
            event.app.exit(result='re-prompt')  # Signal to reprompt.

        @self.add(Keys.F9)
        def handle_f9(event):
            if self.goatshell_instance.prefix == 'az':
                # Call the method on the Goatshell instance to switch Azure subscriptions
                self.goatshell_instance.switch_to_next_subscription()
            elif self.goatshell_instance.prefix == 'goat':
                # Cycle through config stores and update the profile with the returned name
                new_config_store_name = self.goatshell_instance.cycle_config_store()
                # Update the profile to reflect the name of the current config store
                self.goatshell_instance.profile = select_profile(new_config_store_name)
                update_latest_profile(new_config_store_name, self.goatshell_instance.profile)
            else:
                # For other prefixes, cycle through their profiles as before
                self.goatshell_instance.profile = self.goatshell_instance.get_profile(self.goatshell_instance.prefix)
    
            # Refresh UI components to reflect changes
            getLayout()
            self.app.invalidate()
            event.app.exit(result='re-prompt')
    
        @self.add(Keys.F10)
        def handle_f10(event):
            self.goatshell_instance.toggle_vi_mode()
            self.app.invalidate()
            event.app.exit(result='re-prompt')  # Signal to reprompt.

        @self.add(Keys.F12)
        def handle_f12(event):
            self.goatshell_instance.toggle_safety_mode()
            self.app.invalidate()
            event.app.exit(result='re-prompt')  # Signal to reprompt.

def update_latest_profile(config_store, profile_name):
    BASICS = Config(config_store)
    LATEST = BASICS.get_profile('latest')
    if LATEST is None:
        BASICS.create_profile('latest')
        BASICS.update_config(profile_name, 'role', 'latest')
    else:
        BASICS.update_config(profile_name, 'role', 'latest')
